@@ -10,7 +10,7 @@ static const char* FILE_NAME = "Sys/IO/Stat.c";
 #include <sys/stat.h>
 
 #if defined(_WIN32)
-  #include "Sys-Windows.h"
+  #include "spvm_sys_windows.h"
   
   // Exactly same as Perl's one in Win32.h
   #define _S_IFLNK ((unsigned)(_S_IFDIR | _S_IFCHR))
@@ -57,7 +57,7 @@ static time_t file_time_to_epoch(FILETIME file_time) {
   time_t epoch = -1;
   
   if (!FileTimeToSystemTime(&file_time, &system_time)) {
-    win_last_error_to_errno();
+    spvm_sys_windows_win_last_error_to_errno(EINVAL);
     goto END_OF_FUNC;
   }
   
@@ -76,139 +76,165 @@ static time_t file_time_to_epoch(FILETIME file_time) {
 }
 
 // The output data is the same as Perl's win32_stat_low in Win32.c.
-static int win_fstat_by_handle(SPVM_ENV* env, SPVM_VALUE* stack, HANDLE handle, int32_t len, Stat_t *sbuf, DWORD reparse_type) {
-    DWORD type = GetFileType(handle);
-    BY_HANDLE_FILE_INFORMATION bhi;
-    
-    memset(sbuf, 0, 1 * sizeof(Stat_t));
-    
-    if (reparse_type) {
-        /* Lie to get to the right place */
-        type = FILE_TYPE_DISK;
-    }
-
-    type &= ~FILE_TYPE_REMOTE;
-
-    switch (type) {
-    case FILE_TYPE_DISK:
-        if (GetFileInformationByHandle(handle, &bhi)) {
-            sbuf->st_dev = bhi.dwVolumeSerialNumber;
-            sbuf->st_ino = bhi.nFileIndexHigh;
-            sbuf->st_ino <<= 32;
-            sbuf->st_ino |= bhi.nFileIndexLow;
-            sbuf->st_nlink = bhi.nNumberOfLinks;
-            sbuf->st_uid = 0;
-            sbuf->st_gid = 0;
-            /* ucrt sets this to the drive letter for
-               stat(), lets not reproduce that mistake */
-            sbuf->st_rdev = 0;
-            sbuf->st_size = bhi.nFileSizeHigh;
-            sbuf->st_size <<= 32;
-            sbuf->st_size |= bhi.nFileSizeLow;
-
-            sbuf->st_atime = file_time_to_epoch(bhi.ftLastAccessTime);
-            sbuf->st_mtime = file_time_to_epoch(bhi.ftLastWriteTime);
-            sbuf->st_ctime = file_time_to_epoch(bhi.ftCreationTime);
-
-            if (reparse_type) {
-                /* https://docs.microsoft.com/en-us/openspecs/windows_protocols/ms-fscc/c8e77b37-3909-4fe6-a4ea-2b9d423b1ee4
-                   describes all of these as WSL only, but the AF_UNIX tag
-                   is known to be used for AF_UNIX sockets without WSL.
-                */
-                switch (reparse_type) {
-                case IO_REPARSE_TAG_AF_UNIX:
-                    sbuf->st_mode = _S_IFSOCK;
-                    break;
-
-                case IO_REPARSE_TAG_LX_FIFO:
-                    sbuf->st_mode = _S_IFIFO;
-                    break;
-
-                case IO_REPARSE_TAG_LX_CHR:
-                    sbuf->st_mode = _S_IFCHR;
-                    break;
-
-                case IO_REPARSE_TAG_LX_BLK:
-                    sbuf->st_mode = _S_IFBLK;
-                    break;
-
-                default:
-                    /* Is there anything else we can do here? */
-                    errno = EINVAL;
-                    return -1;
-                }
-            }
-            else if (bhi.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
-                sbuf->st_mode = _S_IFDIR | _S_IREAD | _S_IEXEC;
-                /* duplicate the logic from the end of the old win32_stat() */
-                if (!(bhi.dwFileAttributes & FILE_ATTRIBUTE_READONLY)) {
-                    sbuf->st_mode |= S_IWRITE;
-                }
-            }
-            else {
-                WCHAR path_buf_tmp_w[MAX_PATH+1];
-                sbuf->st_mode = _S_IFREG;
-                
-                const char* path = NULL;
-                len = GetFinalPathNameByHandleW(handle, path_buf_tmp_w, sizeof(path_buf_tmp_w), 0);
-                if (len > 0) {
-                  int32_t scope_id = env->enter_scope(env, stack);
-                  
-                  WCHAR* path_buf_w = env->new_memory_block(env, stack, sizeof(WCHAR) * (len + 1));
-                  
-                  len = GetFinalPathNameByHandleW(handle, path_buf_w, len + 1, 0);
-                  
-                  assert(len > 0);
-                  
-                  int32_t error_id = 0;
-                  
-                  path = win_wchar_to_utf8(env, stack, path_buf_w, &error_id, __func__, FILE_NAME, __LINE__);
-                  
-                  env->free_memory_block(env, stack, path_buf_w);
-                  
-                  assert(error_id == 0);
-                  
-                  env->leave_scope(env, stack, scope_id);
-                }
-
-                if (path && len > 4 &&
-                    (_stricmp(path + len - 4, ".exe") == 0 ||
-                     _stricmp(path + len - 4, ".bat") == 0 ||
-                     _stricmp(path + len - 4, ".cmd") == 0 ||
-                     _stricmp(path + len - 4, ".com") == 0)) {
-                    sbuf->st_mode |= _S_IEXEC;
-                }
-                if (!(bhi.dwFileAttributes & FILE_ATTRIBUTE_READONLY)) {
-                    sbuf->st_mode |= _S_IWRITE;
-                }
-                sbuf->st_mode |= _S_IREAD;
-            }
+static int32_t win_fstat_by_handle(SPVM_ENV* env, SPVM_VALUE* stack, HANDLE handle, Stat_t *st_stat) {
+  
+  int32_t status = -1;
+  DWORD type = GetFileType(handle);
+  
+  switch (type) {
+    case FILE_TYPE_DISK: {
+      
+      BY_HANDLE_FILE_INFORMATION file_info = {0};
+      if (GetFileInformationByHandle(handle, &file_info)) {
+        
+        int32_t reparse_type = 0;
+        SPVM_SYS_WINDOWS_REPARSE_DATA_BUFFER linkdata = {0};
+        if (DeviceIoControl(handle, FSCTL_GET_REPARSE_POINT, NULL, 0, &linkdata, sizeof(linkdata), NULL, NULL)) {
+          reparse_type = linkdata.ReparseTag;
         }
         else {
-            win_last_error_to_errno();
-            return -1;
+          if (GetLastError() == ERROR_NOT_A_REPARSE_POINT) {
+            // Do nothing
+          }
+          else {
+            spvm_sys_windows_win_last_error_to_errno(EINVAL);
+            goto END_OF_FUNC;
+          }
         }
-        break;
-
+        
+        st_stat->st_dev = file_info.dwVolumeSerialNumber;
+        st_stat->st_ino = file_info.nFileIndexHigh;
+        st_stat->st_ino <<= 32;
+        st_stat->st_ino |= file_info.nFileIndexLow;
+        st_stat->st_nlink = file_info.nNumberOfLinks;
+        st_stat->st_uid = 0;
+        st_stat->st_gid = 0;
+        /* ucrt sets this to the drive letter for
+           stat(), lets not reproduce that mistake */
+        st_stat->st_rdev = 0;
+        st_stat->st_size = file_info.nFileSizeHigh;
+        st_stat->st_size <<= 32;
+        st_stat->st_size |= file_info.nFileSizeLow;
+        
+        st_stat->st_atime = file_time_to_epoch(file_info.ftLastAccessTime);
+        st_stat->st_mtime = file_time_to_epoch(file_info.ftLastWriteTime);
+        st_stat->st_ctime = file_time_to_epoch(file_info.ftCreationTime);
+        
+        if (reparse_type) {
+          /* https://docs.microsoft.com/en-us/openspecs/windows_protocols/ms-fscc/c8e77b37-3909-4fe6-a4ea-2b9d423b1ee4
+             describes all of these as WSL only, but the AF_UNIX tag
+             is known to be used for AF_UNIX sockets without WSL.
+          */
+          st_stat->st_mode = 0;
+          switch (reparse_type) {
+            case IO_REPARSE_TAG_AF_UNIX: {
+              st_stat->st_mode = _S_IFSOCK;
+              break;
+            }
+            case IO_REPARSE_TAG_LX_FIFO: {
+              st_stat->st_mode = _S_IFIFO;
+              break;
+            }
+            case IO_REPARSE_TAG_LX_CHR: {
+              st_stat->st_mode = _S_IFCHR;
+              break;
+            }
+            case IO_REPARSE_TAG_LX_BLK: {
+              st_stat->st_mode = _S_IFBLK;
+              break;
+            }
+            case IO_REPARSE_TAG_SYMLINK:
+            case IO_REPARSE_TAG_MOUNT_POINT:
+            {
+              break;
+            }
+            default: {
+              /* Is there anything else we can do here? */
+              errno = EINVAL;
+              goto END_OF_FUNC;
+            }
+          }
+        }
+        
+        if (st_stat->st_mode == 0) {
+          if (file_info.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
+            st_stat->st_mode = _S_IFDIR | _S_IREAD | _S_IEXEC;
+            /* duplicate the logic from the end of the old win32_stat() */
+            if (!(file_info.dwFileAttributes & FILE_ATTRIBUTE_READONLY)) {
+              st_stat->st_mode |= S_IWRITE;
+            }
+          }
+          else {
+            st_stat->st_mode = _S_IFREG;
+            
+            int32_t needed_len = GetFinalPathNameByHandleW(handle, NULL, 0, 0);
+            
+            if (needed_len == 0) {
+              spvm_sys_windows_win_last_error_to_errno(EINVAL);
+              goto END_OF_FUNC;
+            }
+            
+            WCHAR* path_w = env->new_memory_block(env, stack, sizeof(WCHAR) * (needed_len + 1));
+            
+            int32_t len = GetFinalPathNameByHandleW(handle, path_w, needed_len + 1, 0);
+            
+            if (len) {
+              if (len > 4 &&
+                (_wcsicmp(path_w + len - 4, L".exe") == 0 ||
+                 _wcsicmp(path_w + len - 4, L".bat") == 0 ||
+                 _wcsicmp(path_w + len - 4, L".cmd") == 0 ||
+                 _wcsicmp(path_w + len - 4, L".com") == 0))
+              {
+                st_stat->st_mode |= _S_IEXEC;
+              }
+            }
+            
+            env->free_memory_block(env, stack, path_w);
+            
+            if (!len) {
+              spvm_sys_windows_win_last_error_to_errno(EINVAL);
+              goto END_OF_FUNC;
+            }
+            
+            if (!(file_info.dwFileAttributes & FILE_ATTRIBUTE_READONLY)) {
+              st_stat->st_mode |= _S_IWRITE;
+            }
+            st_stat->st_mode |= _S_IREAD;
+          }
+        }
+      }
+      else {
+        spvm_sys_windows_win_last_error_to_errno(EINVAL);
+        goto END_OF_FUNC;
+      }
+      break;
+    }
     case FILE_TYPE_CHAR:
     case FILE_TYPE_PIPE:
-        sbuf->st_mode = (type == FILE_TYPE_CHAR) ? _S_IFCHR : _S_IFIFO;
-        if (handle == GetStdHandle(STD_INPUT_HANDLE) ||
-            handle == GetStdHandle(STD_OUTPUT_HANDLE) ||
-            handle == GetStdHandle(STD_ERROR_HANDLE)) {
-            sbuf->st_mode |= _S_IWRITE | _S_IREAD;
-        }
-        break;
-
-    default:
-        return -1;
+    {
+      st_stat->st_mode = (type == FILE_TYPE_CHAR) ? _S_IFCHR : _S_IFIFO;
+      if (handle == GetStdHandle(STD_INPUT_HANDLE) ||
+        handle == GetStdHandle(STD_OUTPUT_HANDLE) ||
+        handle == GetStdHandle(STD_ERROR_HANDLE)) {
+        st_stat->st_mode |= _S_IWRITE | _S_IREAD;
+      }
+      break;
     }
-
-    /* owner == user == group */
-    sbuf->st_mode |= (sbuf->st_mode & 0700) >> 3;
-    sbuf->st_mode |= (sbuf->st_mode & 0700) >> 6;
-
-    return 0;
+    default: {
+      errno = EINVAL;
+      goto END_OF_FUNC;
+    }
+  }
+  
+  /* owner == user == group */
+  st_stat->st_mode |= (st_stat->st_mode & 0700) >> 3;
+  st_stat->st_mode |= (st_stat->st_mode & 0700) >> 6;
+  
+  status = 0;
+  
+  END_OF_FUNC:
+  
+  return status;
 }
 
 static int32_t win_stat(SPVM_ENV* env, SPVM_VALUE* stack, Stat_t *st_stat) {
@@ -219,7 +245,7 @@ static int32_t win_stat(SPVM_ENV* env, SPVM_VALUE* stack, Stat_t *st_stat) {
   
   const char* path = env->get_chars(env, stack, obj_path);
   
-  WCHAR* path_w = utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
+  WCHAR* path_w = spvm_sys_windows_utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
   if (error_id) {
     return error_id;
   }
@@ -243,28 +269,21 @@ static int32_t win_stat(SPVM_ENV* env, SPVM_VALUE* stack, Stat_t *st_stat) {
     }
     const char* resolved_link_text = env->get_chars(env, stack, obj_resolved_link_text);
     
-    WCHAR* resolved_link_text_w = utf8_to_win_wchar(env, stack, resolved_link_text, &error_id, __func__, FILE_NAME, __LINE__);
+    WCHAR* resolved_link_text_w = spvm_sys_windows_utf8_to_win_wchar(env, stack, resolved_link_text, &error_id, __func__, FILE_NAME, __LINE__);
     if (error_id) {
       return error_id;
     }
     
-    handle = CreateFileW_reparse_point_for_read(resolved_link_text_w);
+    handle = spvm_sys_windows_CreateFileW_reparse_point_for_read(resolved_link_text_w);
     
     if (handle == INVALID_HANDLE_VALUE) {
-      win_last_error_to_errno();
+      spvm_sys_windows_win_last_error_to_errno(EINVAL);
       error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
       goto END_OF_FUNC;
     }
-    
-    MY_REPARSE_DATA_BUFFER linkdata;
-    DWORD linkdata_returned;
-    
-    if (DeviceIoControl(handle, FSCTL_GET_REPARSE_POINT, NULL, 0, &linkdata, sizeof(linkdata), &linkdata_returned, NULL)) {
-      ReparseTag = linkdata.ReparseTag;
-    }
   }
   
-  int32_t result = win_fstat_by_handle(env, stack, handle, 0, st_stat, ReparseTag);
+  int32_t result = win_fstat_by_handle(env, stack, handle, st_stat);
   
   if (result == -1) {
     error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
@@ -296,26 +315,26 @@ static int32_t win_lstat(SPVM_ENV* env, SPVM_VALUE* stack, Stat_t *st_stat) {
   
   const char* path = env->get_chars(env, stack, obj_path);
   
-  WCHAR* path_w = utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
+  WCHAR* path_w = spvm_sys_windows_utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
   if (error_id) {
     goto END_OF_FUNC;
   }
   
-  HANDLE handle = CreateFileW_reparse_point_for_read(path_w);
+  HANDLE handle = spvm_sys_windows_CreateFileW_reparse_point_for_read(path_w);
   if (handle == INVALID_HANDLE_VALUE) {
-    win_last_error_to_errno();
+    spvm_sys_windows_win_last_error_to_errno(EINVAL);
     error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
     goto END_OF_FUNC;
   }
   
-  int32_t result = win_fstat_by_handle(env, stack, handle, 0, st_stat, 0);
+  int32_t result = win_fstat_by_handle(env, stack, handle, st_stat);
   
   if (result == -1) {
     error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
     goto END_OF_FUNC;
   }
   
-  int32_t is_sym = is_symlink_by_handle(handle);
+  int32_t is_sym = spvm_sys_windows_is_symlink_by_handle(handle);
   
   if (is_sym) {
     void* obj_link_text = NULL;
@@ -397,14 +416,14 @@ int32_t SPVM__Sys__IO__Stat__stat(SPVM_ENV* env, SPVM_VALUE* stack) {
     return env->die(env, stack, "The stat object $stat must be defined.", __func__, FILE_NAME, __LINE__);
   }
   
-  Stat_t* stat_buf = env->get_pointer(env, stack, obj_stat);
+  Stat_t* st_stat = env->get_pointer(env, stack, obj_stat);
   
 #if defined(_WIN32)
   stack[0].oval = obj_path;
-  error_id = win_stat(env, stack, stat_buf);
+  error_id = win_stat(env, stack, st_stat);
   int32_t status = error_id ? -1 : 0;
 #else
-  int32_t status = stat(path, stat_buf);
+  int32_t status = stat(path, st_stat);
 #endif
   
   if (status == -1) {
@@ -435,14 +454,14 @@ int32_t SPVM__Sys__IO__Stat__lstat(SPVM_ENV* env, SPVM_VALUE* stack) {
     return env->die(env, stack, "The stat object $lstat must be defined.", __func__, FILE_NAME, __LINE__);
   }
   
-  Stat_t* stat_buf = env->get_pointer(env, stack, obj_lstat);
+  Stat_t* st_stat = env->get_pointer(env, stack, obj_lstat);
   
 #if defined(_WIN32)
   stack[0].oval = obj_path;
-  error_id = win_lstat(env, stack, stat_buf);
+  error_id = win_lstat(env, stack, st_stat);
   int32_t status = error_id ? -1 : 0;
 #else
-  int32_t status = lstat(path, stat_buf);
+  int32_t status = lstat(path, st_stat);
 #endif
 
   if (status == -1) {
@@ -468,14 +487,14 @@ int32_t SPVM__Sys__IO__Stat__fstat(SPVM_ENV* env, SPVM_VALUE* stack) {
     return env->die(env, stack, "The stat object $stat must be defined.", __func__, FILE_NAME, __LINE__);
   }
   
-  Stat_t* stat_buf = env->get_pointer(env, stack, obj_stat);
+  Stat_t* st_stat = env->get_pointer(env, stack, obj_stat);
   
   
 #if defined(_WIN32)
   HANDLE handle = (HANDLE)_get_osfhandle(fd);
-  int32_t status = win_fstat_by_handle(env, stack, handle, 0, stat_buf, 0);
+  int32_t status = win_fstat_by_handle(env, stack, handle, st_stat);
 #else
-  int32_t status = fstat(fd, stat_buf);
+  int32_t status = fstat(fd, st_stat);
 #endif
 
   if (status == -1) {
